/*!
 * Copyright (C) 2001-2005 by EmbeddedIT, 
 * Ole Reinhardt <ole.reinhardt@embedded-it.de> All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EMBEDDED IT AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EMBEDDED IT
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: fligh-ctrl.c,v $
 */

#include <cfg/crt.h>    /* Floating point configuration. */

#include <string.h>
#include <stdio.h>
#include <io.h>
#include <stdlib.h>

#include <dev/board.h>
#include <dev/usartat91.h>
#include <dev/at91_spi.h>
#include <dev/gpio.h>

#include <sys/timer.h>
#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/socket.h>
#include <sys/confnet.h>

#include <arpa/inet.h>
#include <pro/dhcp.h>

#include "nokia-lcd.h"
#include "lcd.h"
#include "bmp.h"
#include "drawing.h"

#define BUFFERSIZE  128
#define TCPPORT     23

#define   LCD_RESET_PORT        1   // PIOA
#define   LCD_RESET_PIN         2

#define   LCD_BACKLIGHT_PORT    2   // PIOB
#define   LCD_BACKLIGHT_PIN     20  

typedef struct {
    FILE *cd_rs232;
    FILE *cd_tcpip;
    char cd_connected;
} CHANNEL;


void WriteLcdCommand(unsigned int data_out)
{
    unsigned int data_in;
    
    data_out = data_out & ~0x0100;
    At91SpiTransfer2(SPI0_BASE, 0, &data_out, &data_in, 1, NULL, NULL, 0);
}

void WriteLcdData(unsigned int data_out)
{
    unsigned int data_in;

    data_out = data_out | 0x0100;
    At91SpiTransfer2(SPI0_BASE, 0, &data_out, &data_in, 1, NULL, NULL, 0);
}

void LCDReset(void)
{
    GpioPinSetLow(LCD_RESET_PORT, LCD_RESET_PIN);
    NutSleep(10);    
    GpioPinSetHigh(LCD_RESET_PORT, LCD_RESET_PIN);
    NutSleep(10);    
}

void LCDSettings(void) 
{
    // Hardware reset
    LCDReset();
    
    WriteLcdCommand(DISCTL);
    WriteLcdData(0x00); // default
    WriteLcdData(0x20); // (32 + 1) * 4 = 132 lines (of which 130 are visible)
    WriteLcdData(0x0a); // default

    // COM scan
    WriteLcdCommand(COMSCN);
    WriteLcdData(0x00);  // Scan 1-80

    // Internal oscilator ON
    WriteLcdCommand(OSCON);

    // Sleep out
    WriteLcdCommand(SLPOUT);

    // Voltage control
    WriteLcdCommand(VOLCTR);
    WriteLcdData(0x1F); // middle value of V1
    WriteLcdData(0x03); // middle value of resistance value

    // Temperature gradient
    WriteLcdCommand(TMPGRD);
    WriteLcdData(0x00); // default

    // Power control
    WriteLcdCommand(PWRCTR);
    WriteLcdData(0x0f);   // referance voltage regulator on, circuit voltage follower on, BOOST ON

    // Normal display
    WriteLcdCommand(DISNOR);

    // Inverse display
    WriteLcdCommand(DISINV);

    // Partial area off
    WriteLcdCommand(PTLOUT);

    // Data control
    WriteLcdCommand(DATCTL);
    WriteLcdData(0x00); // all inversions off, column direction
    WriteLcdData(0x03); // RGB sequence
    WriteLcdData(0x02); // Grayscale -> 16

    // Page Address set
    WriteLcdCommand(PASET);
    WriteLcdData(0);
    WriteLcdData(131);

    // Page Column set
    WriteLcdCommand(CASET);
    WriteLcdData(0);
    WriteLcdData(131);
}


void LCDWrite130x130bmp_bytewise(void) 
{
    int idx;
    // Display OFF
    //WriteLcdCommand(DISOFF);

    // WRITE MEMORY
    WriteLcdCommand(RAMWR);

    for(idx = 0; idx < 25740; idx++) {
        WriteLcdData(bmp[idx]);
    }

    // wait aproximetly 100ms
    //NutDelay(100);

    // Display On
    //WriteLcdCommand(DISON);
}

/*
void LCDWrite130x130bmp_block(void) 
{
    int idx;
    u_short *rcv;
    
    img = malloc(sizeof(bmp) * 2);
    rcv = malloc(sizeof(bmp) * 2);
        
    // Display OFF
    WriteLcdCommand(DISOFF);

    // WRITE MEMORY
    WriteLcdCommand(RAMWR);

    for(idx = 0; idx < 25740; idx++) {
        //img[idx] = bmp[idx] | 0x0100;
        img[idx] = (idx & 0xFF) | 0x0100;
    }
    
    
    At91SpiTransfer2(SPI0_BASE, 0, img, rcv, sizeof(bmp), NULL, NULL, 0);    
    // Display On
    WriteLcdCommand(DISON);	
    free(img);
    free(rcv);
}
*/

void LCDSPISetup(void)
{
    At91SpiInit(SPI0_BASE);
    At91SpiInitChipSelects(SPI0_BASE, _BV(0));
    
    //At91SpiSetRate(SPI0_BASE, 0, 166000);
    At91SpiSetRate(SPI0_BASE, 0, 20000000);
    At91SpiSetModeFlags(SPI0_BASE, 0, SPIMF_MASTER | SPIMF_MFDETECT | SPIMF_SCKIAHI | SPIMF_CAPRISE);
    At91SpiSetBits(SPI0_BASE, 0, 9);
    At91SpiSetSckDelay(SPI0_BASE, 0, 1);
    At91SpiSetTxDelay(SPI0_BASE, 0, 1);
    
    //printf("SPI_CSR %08x, SPI_MR %08x\r\n", inr(SPI0_BASE + SPI_CSR0_OFF), inr(SPI0_BASE + SPI_MR_OFF));
    //printf_P(PSTR("Spi init ok..." NL)); 
    
}

THREAD(Receiver, arg)
{
CHANNEL *cdp = arg;

    for (;;) {
        if (cdp->cd_connected) {
            NutThreadSetPriority(64);
        }
        NutThreadYield();
    }
}


int main(void)
{
    u_long baud = 115200;
    FILE *uart;
    TCPSOCKET *sock;
    CHANNEL cd;
    int i;
	
	int x=0;
	int y=0;
	int z=0;
	int c1=0;	//color red
	int c2=0;	//color green
	int c3=0;	//color blue
	
//    unsigned int lcd_colors[] = {0xFFF, 0x000, 0xF00, 0x0F0, 0x00F, 0x0FF, 0xF0F, 0xFF0, 0xB22, 0xFA0, 0xF6A };
//    unsigned char buf[40];

    /* Register and configure devices */
    NutRegisterDevice(&DEV_UART, 0, 0);

    uart = fopen(DEV_UART_NAME, "r+");
    freopen(DEV_UART_NAME, "w", stdout);
    
    _ioctl(_fileno(uart), UART_SETSPEED, &baud);

    printf_P(PSTR("Olimex SAM7X" NL));

    /* Enable lcd backlight pin PB20 as output */
    GpioPinConfigSet(LCD_BACKLIGHT_PORT, LCD_BACKLIGHT_PIN, GPIO_CFG_OUTPUT);   
    GpioPinSetHigh(LCD_BACKLIGHT_PORT, LCD_BACKLIGHT_PIN);
    
    /* Enable reset pin PA2 as output */
    GpioPinConfigSet(LCD_RESET_PORT, LCD_RESET_PIN, GPIO_CFG_OUTPUT);   
    GpioPinSetHigh(LCD_RESET_PORT, LCD_RESET_PIN);
    
    LCDSPISetup();

    // init LCD
    LCDSettings();

    // LCD on
    WriteLcdCommand(DISON);

    NutSleep(2000);
	LCDClearScreen(WHITE);
    NutSleep(2000);
	LCDClearScreen(RED);
    NutSleep(2000);
	LCDClearScreen(GREEN);
    NutSleep(2000);
	LCDClearScreen(BLUE);

    NutSleep(2000);
	LCDClearScreen(CYAN);//ligt blue
    NutSleep(2000);
	LCDClearScreen(MAGENTA);//rose
    NutSleep(2000);
	LCDClearScreen(YELLOW);

    NutSleep(2000);
	LCDClearScreen(BROWN);
    NutSleep(2000);
	LCDClearScreen(ORANGE);//rose
    NutSleep(2000);
	LCDClearScreen(PINK);
    NutSleep(2000);

	LCDClearScreen(BLACK);
    NutSleep(2000);

	// color demo by Ronald Lokker

	for (z=0;z<64;z++)
	{
		for (y=0;y<64;y++)
		{
			for (x=0;x<64;x++)
			{
				c1=((x>>2) & 0x000F)  ;
				c2=(((y>>2)<<4) & 0x00F0);
				c3=(((z>>2)<<8) & 0x0F00);
				LCDSetPixel(x+z,y+z,c1+c2+c3);
			}
		}
	}

	NutSleep(5000);

	for (z=0;z<64;z++)
	{
		for (y=0;y<64;y++)
		{
			for (x=0;x<64;x++)
			{
				c1=((z>>2) & 0x000F)  ;		// Red
				c2=(((y>>2)<<4) & 0x00F0);		// Green
				c3=(((x>>2)<<8) & 0x0F00);		// Blue
				LCDSetPixel(128-x-z,y+z,c1+c2+c3);
			}
		}
	}

    NutSleep(10000);

   // display startup image
    LCDWrite130x130bmp_bytewise();
    NutSleep(50000);
    LCDClearScreen(WHITE);

    LCDPutStr("NetConfig ", 01, 10, MEDIUM, BLACK, WHITE);

    NutRegisterDevice(&DEV_ETHER, 0x8300, 5);

    if (NutDhcpIfConfig(DEV_ETHER_NAME, 0, 60000)) {
        /* No valid EEPROM contents, use hard coded MAC. */
    	u_char my_mac[] = { 0x00, 0x06, 0x98, 0x20, 0x00, 0x00 };
    	if (NutDhcpIfConfig("eth0", my_mac, 60000)) {
            u_long ip_addr = inet_addr("192.168.192.100");
            u_long ip_mask = inet_addr("255.255.255.0");
            NutNetIfConfig("eth0", my_mac, ip_addr, ip_mask);
            LCDPutStr("DHCP failed ", 01, 10, MEDIUM, BLACK, WHITE);
            LCDPutStr("192.168.192.100", 01, 20, MEDIUM, BLACK, WHITE);
        }
    } else {
        LCDPutStr("DHCP OK     ", 01, 10, MEDIUM, BLACK, WHITE);
        LCDPutStr(inet_ntoa(confnet.cdn_ip_addr), 01, 20, MEDIUM, BLACK, WHITE);
    }

    cd.cd_connected = 0;

    //NutThreadCreate("xmit", Receiver, &cd, 512);

    while (1) {
        sock = NutTcpCreateSocket();
        NutTcpAccept(sock, TCPPORT);
        cd.cd_tcpip = _fdopen((int) sock, "r+b");
//        cd.cd_connected = 1;

        i=0;
/*		
        while (cd.cd_connected) {
#if 0
            if (fprintf(cd.cd_tcpip, "%d\n", i++) <= 0)
            		cd.cd_connected=0;
            if (cd.cd_connected && fflush(cd.cd_tcpip))
            		cd.cd_connected=0;
            NutSleep(1000);
#endif
            if (cd.cd_connected && fprintf(cd.cd_tcpip, "For help press 'h': ") <= 0)
                cd.cd_connected=0;
            if (cd.cd_connected && fflush(cd.cd_tcpip))
                cd.cd_connected=0;
            if (cd.cd_connected && (fread(buf, 1, 1, cd.cd_tcpip)) <= 0)
                cd.cd_connected=0;
            if (cd.cd_connected && fprintf(cd.cd_tcpip, "%c \n", buf[0]) <= 0)
                cd.cd_connected=0;
            if (cd.cd_connected && fflush(cd.cd_tcpip))
                cd.cd_connected=0;
            switch (buf[0]) {
                case 'c':
                        LCDClearScreen(WHITE);
                        break;
                case 'h':
                        fprintf(cd.cd_tcpip, " c - clear LCD\n");
                        fprintf(cd.cd_tcpip, " h - help, show this help\n");
                        fprintf(cd.cd_tcpip, " q - quit and close connection\n");
                        fprintf(cd.cd_tcpip, " t - print text on LCD\n");
                        fflush(cd.cd_tcpip);
                        break;
                case 'q':
                        fprintf(cd.cd_tcpip, "\nQuitting, closing connection...\n");
                        fflush(cd.cd_tcpip);
                        cd.cd_connected=0;
                        break;
                case 't':
                        LCDPutStr("Hello World!", 5, 10, MEDIUM, BLACK, WHITE);
                        break;
                default:
                        fprintf(cd.cd_tcpip, "\n ? unknown command, press 'h' for help\n");
                        fflush(cd.cd_tcpip);
                        break;
            }
        }
        fclose(cd.cd_tcpip);
        NutTcpCloseSocket(sock);
*/        
#if 0
        LCDSetLine(x1,y1,x2,y2,lcd_colors[c]);
        x1+=x1inc; if (x1>=128 || x1<=0) { x1inc*=-1; c++; c%=11; };
        x2+=x1inc; if (x2>=128 || x2<=0) x2inc*=-1;
        y1+=y1inc; if (y1>=128 || y1<=0) y1inc*=-1;
        y2+=x1inc; if (y2>=128 || y2<=0) y2inc*=-1;
#endif
    } 
}
